"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ByteList = exports.DataTypes = void 0;
var DataTypes;
(function (DataTypes) {
    DataTypes[DataTypes["BYTE"] = 0] = "BYTE";
    DataTypes[DataTypes["BOOL"] = 1] = "BOOL";
    DataTypes[DataTypes["INT8"] = 2] = "INT8";
    DataTypes[DataTypes["INT16"] = 3] = "INT16";
    DataTypes[DataTypes["INT32"] = 4] = "INT32";
    DataTypes[DataTypes["UINT8"] = 5] = "UINT8";
    DataTypes[DataTypes["UINT16"] = 6] = "UINT16";
    DataTypes[DataTypes["UINT32"] = 7] = "UINT32";
    DataTypes[DataTypes["FLOAT"] = 8] = "FLOAT";
    DataTypes[DataTypes["DOUBLE"] = 9] = "DOUBLE";
    DataTypes[DataTypes["DATE"] = 10] = "DATE";
    DataTypes[DataTypes["STRING"] = 11] = "STRING";
    DataTypes[DataTypes["BYTE_ARRAY"] = 12] = "BYTE_ARRAY";
})(DataTypes = exports.DataTypes || (exports.DataTypes = {}));
// TODO: Address the following outstanding issues.
//      1.  There is not way to change the initial padding.  It should parameter in the constructor
//      2.  toSting prints all of the allocated space
//      3.  BYTE_ARRAYs are encoded with a byte length, but don't prevent writing larger arrays
var ByteList = /** @class */ (function () {
    function ByteList(bytes) {
        this.useLittleEndian = true;
        this._paddingSize = 100;
        this.index = 0;
        this._length = 0;
        this._buffer = Buffer.alloc(this._paddingSize);
        if (typeof bytes === 'string' || bytes instanceof String) {
            this.writeString(bytes);
        }
        else if (bytes) {
            this.concat(bytes);
        }
        this.index = 0;
        return this;
    }
    ByteList.SetBit = function (num, val, bit) {
        if (bit < 0 || bit > 31) {
            return num >>> 0;
        }
        if (val) {
            num |= 1 << bit;
        }
        else {
            num &= ~(1 << bit);
        }
        return num >>> 0;
    };
    ByteList.GetBit = function (num, bit) {
        if (bit < 0 || bit > 31) {
            return false;
        }
        return (num & (1 << bit)) != 0;
    };
    Object.defineProperty(ByteList.prototype, "length", {
        get: function () { return this._length; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ByteList.prototype, "paddingSize", {
        get: function () { return this._paddingSize; },
        set: function (val) {
            this._paddingSize = val;
            if (this._paddingSize < 0) {
                this._paddingSize = 0;
            }
        },
        enumerable: false,
        configurable: true
    });
    ByteList.prototype.getBuffer = function () {
        return this._buffer.slice(0, this._length);
    };
    ByteList.prototype.getLength = function () {
        return this._length;
    };
    ByteList.prototype.concat = function (bytes) {
        var length;
        var buffer;
        if (bytes instanceof ArrayBuffer) {
            buffer = new Uint8Array(bytes);
            length = buffer.length;
        }
        else if (bytes instanceof ByteList) {
            buffer = bytes.getBuffer();
            length = bytes.length;
        }
        else {
            buffer = bytes;
            length = bytes.length;
        }
        if (buffer && buffer.length) {
            this.prepareBuffer(length);
            for (var i = 0; i < length; i++) {
                if (this.index === this._length) {
                    this.index++;
                }
                this._buffer.writeUInt8(buffer[i] & 0xFF, this._length++);
            }
        }
    };
    ByteList.prototype.insert = function (buffer) {
        if (buffer instanceof ByteList) {
            buffer = buffer.getBuffer();
        }
        var part1 = this._buffer.slice(0, this.index);
        var part2 = this._buffer.slice(this.index, this._buffer.length);
        this._buffer = Buffer.concat([part1, buffer, part2], (part1.length + buffer.length + part2.length));
        this.index += buffer.length;
        this._length += buffer.length;
    };
    ByteList.prototype.peekByte = function (offset) {
        if (offset === void 0) { offset = 0; }
        if (this.length < this.index + offset + 1) {
            throw new Error('Buffer Overrun');
        }
        return this._buffer.readUInt8(this.index + offset);
    };
    ByteList.prototype.peekUInt16 = function (offset) {
        if (offset === void 0) { offset = 0; }
        if (this.length < this.index + offset + 2) {
            throw new Error('Buffer Overrun');
        }
        if (this.useLittleEndian) {
            return this._buffer.readUInt16LE(this.index + offset);
        }
        else {
            return this._buffer.readUInt16BE(this.index + offset);
        }
    };
    ByteList.prototype.peekUInt32 = function (offset) {
        if (offset === void 0) { offset = 0; }
        if (this.length < this.index + offset + 4) {
            throw new Error('Buffer Overrun');
        }
        if (this.useLittleEndian) {
            return this._buffer.readUInt32LE(this.index + offset);
        }
        else {
            return this._buffer.readUInt32BE(this.index + offset);
        }
    };
    ByteList.prototype.writeByte = function (byte, options) {
        if (options === void 0) { options = {}; }
        if (typeof byte === 'string' && byte.length === 1) {
            byte = byte.charCodeAt(0);
        }
        this.prepareBuffer(1);
        var buf = options.insert ? Buffer.alloc(1) : this._buffer;
        buf.writeUInt8(!byte || byte < 0 ? 0 : byte & 0xFF, options.insert ? 0 : this.index);
        if (options.insert) {
            this.insert(buf);
        }
        else {
            this.index += 1;
            this._length += 1;
        }
    };
    ByteList.prototype.writeInt8 = function (int8, options) {
        if (options === void 0) { options = {}; }
        this.prepareBuffer(1);
        var buf = options.insert ? Buffer.alloc(1) : this._buffer;
        buf.writeInt8(!int8 ? 0 : int8, options.insert ? 0 : this.index);
        if (options.insert) {
            this.insert(buf);
        }
        else {
            this.index += 1;
            this._length += 1;
        }
    };
    ByteList.prototype.writeBool = function (bool, options) {
        if (options === void 0) { options = {}; }
        this.writeByte(bool ? 1 : 0, options);
    };
    ByteList.prototype.writeInt16 = function (int16, options) {
        if (options === void 0) { options = {}; }
        this.prepareBuffer(2);
        var buf = options.insert ? Buffer.alloc(2) : this._buffer;
        if (this.useLittleEndian) {
            buf.writeInt16LE(!int16 ? 0 : int16 & 0xFFFF, options.insert ? 0 : this.index);
        }
        else {
            buf.writeInt16BE(!int16 ? 0 : int16 & 0xFFFF, options.insert ? 0 : this.index);
        }
        if (options.insert) {
            this.insert(buf);
        }
        else {
            this.index += 2;
            this._length += 2;
        }
    };
    ByteList.prototype.writeInt32 = function (int32, options) {
        if (options === void 0) { options = {}; }
        this.prepareBuffer(4);
        var buf = options.insert ? Buffer.alloc(4) : this._buffer;
        if (this.useLittleEndian) {
            buf.writeInt32LE(!int32 ? 0 : (int32 & 0xFFFFFFFF) >>> 0, options.insert ? 0 : this.index);
        }
        else {
            buf.writeInt32BE(!int32 ? 0 : (int32 & 0xFFFFFFFF) >>> 0, options.insert ? 0 : this.index);
        }
        if (options.insert) {
            this.insert(buf);
        }
        else {
            this.index += 4;
            this._length += 4;
        }
    };
    ByteList.prototype.writeUInt16 = function (uint16, options) {
        if (options === void 0) { options = {}; }
        this.prepareBuffer(2);
        var buf = options.insert ? Buffer.alloc(2) : this._buffer;
        if (this.useLittleEndian) {
            buf.writeUInt16LE(!uint16 ? 0 : uint16 & 0xFFFF, options.insert ? 0 : this.index);
        }
        else {
            buf.writeUInt16BE(!uint16 ? 0 : uint16 & 0xFFFF, options.insert ? 0 : this.index);
        }
        if (options.insert) {
            this.insert(buf);
        }
        else {
            this.index += 2;
            this._length += 2;
        }
    };
    ByteList.prototype.writeUInt32 = function (uint32, options) {
        if (options === void 0) { options = {}; }
        this.prepareBuffer(4);
        var buf = options.insert ? Buffer.alloc(4) : this._buffer;
        if (this.useLittleEndian) {
            buf.writeUInt32LE(!uint32 ? 0 : (uint32 & 0xFFFFFFFF) >>> 0, options.insert ? 0 : this.index);
        }
        else {
            buf.writeUInt32BE(!uint32 ? 0 : (uint32 & 0xFFFFFFFF) >>> 0, options.insert ? 0 : this.index);
        }
        if (options.insert) {
            this.insert(buf);
        }
        else {
            this.index += 4;
            this._length += 4;
        }
    };
    ByteList.prototype.writeFloat = function (float, options) {
        if (options === void 0) { options = {}; }
        this.prepareBuffer(8);
        var buf = options.insert ? Buffer.alloc(8) : this._buffer;
        if (this.useLittleEndian) {
            buf.writeFloatLE(!float ? 0 : float, options.insert ? 0 : this.index);
        }
        else {
            buf.writeFloatBE(!float ? 0 : float, options.insert ? 0 : this.index);
        }
        if (options.insert) {
            this.insert(buf);
        }
        else {
            this.index += 8;
            this._length += 8;
        }
    };
    ByteList.prototype.writeDouble = function (double, options) {
        if (options === void 0) { options = {}; }
        this.prepareBuffer(8);
        var buf = options.insert ? Buffer.alloc(8) : this._buffer;
        if (this.useLittleEndian) {
            buf.writeDoubleLE(!double ? 0 : double, options.insert ? 0 : this.index);
        }
        else {
            buf.writeDoubleBE(!double ? 0 : double, options.insert ? 0 : this.index);
        }
        if (options.insert) {
            this.insert(buf);
        }
        else {
            this.index += 8;
            this._length += 8;
        }
    };
    ByteList.prototype.writeDate = function (date, options) {
        if (options === void 0) { options = {}; }
        this.prepareBuffer(6);
        var buffer = options.insert ? Buffer.alloc(6) : this._buffer;
        buffer.writeUInt8(!date ? 0 : (date.getUTCFullYear() - 2000) & 0xFF, options.insert ? 0 : this.index++);
        buffer.writeUInt8(!date ? 0 : date.getUTCMonth() & 0xFF, options.insert ? 1 : this.index++);
        buffer.writeUInt8(!date ? 0 : date.getUTCDate() & 0xFF, options.insert ? 2 : this.index++);
        buffer.writeUInt8(!date ? 0 : date.getUTCHours() & 0xFF, options.insert ? 3 : this.index++);
        buffer.writeUInt8(!date ? 0 : date.getUTCMinutes() & 0xFF, options.insert ? 4 : this.index++);
        buffer.writeUInt8(!date ? 0 : date.getUTCSeconds() & 0xFF, options.insert ? 5 : this.index++);
        if (options.insert) {
            this.insert(buffer);
        }
        else {
            this._length += 6;
        }
    };
    ByteList.prototype.writeString = function (str, options) {
        if (str === void 0) { str = ''; }
        if (options === void 0) { options = {}; }
        if (!options.length) {
            var buf = Buffer.from(str, 'ascii');
            this.writeUInt16(buf.length, options);
            this.concat(buf);
        }
        else {
            var buf = Buffer.from(str.length > options.length ? str.substr(0, options.length) : str, 'ascii');
            console.log(buf);
            this.concat(buf);
            for (var i = 0; i < options.length - str.length; i++) {
                this.writeByte(0);
            }
        }
    };
    ByteList.prototype.writeByteArray = function (list, options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        this.writeByte(list ? list.length : 0, options);
        (list || []).forEach(function (l) {
            _this.writeByte(l, options);
        });
    };
    ByteList.prototype.writeData = function (type, val, options) {
        switch (type) {
            case DataTypes.BYTE:
                this.writeByte(val, options);
                break;
            case DataTypes.BOOL:
                this.writeBool(val, options);
                break;
            case DataTypes.INT8:
                this.writeByte(val, options);
                break;
            case DataTypes.INT16:
                this.writeInt16(val, options);
                break;
            case DataTypes.INT32:
                this.writeInt32(val, options);
                break;
            case DataTypes.UINT8:
                this.writeByte(val, options);
                break;
            case DataTypes.UINT16:
                this.writeUInt16(val, options);
                break;
            case DataTypes.UINT32:
                this.writeUInt32(val, options);
                break;
            case DataTypes.FLOAT:
                this.writeFloat(val, options);
                break;
            case DataTypes.DOUBLE:
                this.writeDouble(val, options);
                break;
            case DataTypes.DATE:
                this.writeDate(val, options);
                break;
            case DataTypes.STRING:
                this.writeString(val, options);
                break;
            case DataTypes.BYTE_ARRAY:
                this.writeByteArray(val, options);
                break;
        }
    };
    ByteList.prototype.trimLeft = function (count) {
        if (count <= 0) {
            return Buffer.from('');
        }
        if (count > this._length) {
            count = this._length;
        }
        var bytes = this._buffer.slice(0, count);
        this._buffer = this._buffer.slice(count, this._buffer.length);
        this.index = this.index - count;
        if (this.index < 0) {
            this.index = 0;
        }
        this._length -= count;
        return bytes;
    };
    ByteList.prototype.trimRight = function (count) {
        if (count <= 0) {
            return Buffer.from('');
        }
        if (count > this._length) {
            count = this._length;
        }
        var bytes = this._buffer.slice(this._length - count, this._length);
        this._length -= count;
        if (this.index > this._length) {
            this.index = this._length;
        }
        return bytes;
    };
    ByteList.prototype.readByte = function () {
        if (this.length < this.index + 1) {
            throw new Error('Buffer Overrun');
        }
        var byte = this._buffer.readUInt8(this.index);
        this.index++;
        return byte;
    };
    ByteList.prototype.readInt8 = function () {
        if (this.length < this.index + 1) {
            throw new Error('Buffer Overrun');
        }
        var byte = this._buffer.readInt8(this.index);
        this.index++;
        return byte;
    };
    ByteList.prototype.readBool = function () {
        return this.readByte() === 1;
    };
    ByteList.prototype.readInt16 = function () {
        if (this.length < this.index + 2) {
            throw new Error('Buffer Overrun');
        }
        var val = this.useLittleEndian ? this._buffer.readInt16LE(this.index) : this._buffer.readInt16BE(this.index);
        this.index += 2;
        return val;
    };
    ByteList.prototype.readInt32 = function () {
        if (this.length < this.index + 4) {
            throw new Error('Buffer Overrun');
        }
        var val = this.useLittleEndian ? this._buffer.readInt32LE(this.index) : this._buffer.readInt32BE(this.index);
        this.index += 4;
        return val;
    };
    ByteList.prototype.readUInt16 = function () {
        if (this.length < this.index + 2) {
            throw new Error('Buffer Overrun');
        }
        var val = this.useLittleEndian ? this._buffer.readUInt16LE(this.index) : this._buffer.readUInt16BE(this.index);
        this.index += 2;
        return val;
    };
    ByteList.prototype.readUInt32 = function () {
        if (this.length < this.index + 4) {
            throw new Error('Buffer Overrun');
        }
        var val = this.useLittleEndian ? this._buffer.readUInt32LE(this.index) : this._buffer.readUInt32BE(this.index);
        this.index += 4;
        return val;
    };
    ByteList.prototype.readFloat = function () {
        if (this.length < this.index + 8) {
            throw new Error('Buffer Overrun');
        }
        var val = this.useLittleEndian ? this._buffer.readFloatLE(this.index) : this._buffer.readFloatBE(this.index);
        this.index += 8;
        return val;
    };
    ByteList.prototype.readDouble = function () {
        if (this.length < this.index + 8) {
            throw new Error('Buffer Overrun');
        }
        var val = this.useLittleEndian ? this._buffer.readDoubleLE(this.index) : this._buffer.readDoubleBE(this.index);
        this.index += 8;
        return val;
    };
    ByteList.prototype.readDate = function () {
        try {
            var year = this.readByte() + 2000;
            var month = this.readByte();
            var day = this.readByte();
            var hour = this.readByte();
            var minute = this.readByte();
            var second = this.readByte();
            return new Date(Date.UTC(year, month, day, hour, minute, second));
        }
        catch (e) {
            return null;
        }
    };
    ByteList.prototype.readBytes = function (count) {
        if (this.length < this.index + count) {
            throw new Error('Buffer Overrun');
        }
        var bytes = this._buffer.slice(this.index, this.index + count);
        this.index += count;
        return bytes;
    };
    ByteList.prototype.readString = function (options) {
        if (options === void 0) { options = {}; }
        if (this.length < this.index + 2) {
            throw new Error('Buffer Overrun');
        }
        var length = options.length || this.readUInt16();
        if (this.length < this.index + length) {
            throw new Error('Buffer Overrun');
        }
        var str = this._buffer.toString('ascii', this.index, this.index + length).replace(/\0/g, '');
        this.index += length;
        return str;
    };
    ByteList.prototype.readByteArray = function () {
        var array = [];
        var length = this.readByte();
        for (var i = 0; i < length; i++) {
            array.push(this.readByte());
        }
        return array;
    };
    ByteList.prototype.readData = function (type, options) {
        if (options === void 0) { options = {}; }
        switch (type) {
            case DataTypes.BYTE:
                return this.readByte();
            case DataTypes.BOOL:
                return this.readBool();
            case DataTypes.INT8:
                return this.readInt8();
            case DataTypes.INT16:
                return this.readInt16();
            case DataTypes.INT32:
                return this.readInt32();
            case DataTypes.UINT8:
                return this.readByte();
            case DataTypes.UINT16:
                return this.readUInt16();
            case DataTypes.UINT32:
                return this.readUInt32();
            case DataTypes.FLOAT:
                return this.readFloat();
            case DataTypes.DOUBLE:
                return this.readDouble();
            case DataTypes.DATE:
                return this.readDate();
            case DataTypes.STRING:
                return this.readString(options);
            case DataTypes.BYTE_ARRAY:
                return this.readByteArray();
        }
    };
    ByteList.prototype.toString = function () {
        var e_1, _a;
        var str = '';
        try {
            for (var _b = __values(this._buffer), _c = _b.next(); !_c.done; _c = _b.next()) {
                var byte = _c.value;
                str += byte.toString(16).toUpperCase() + ' ';
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return str;
    };
    ByteList.prototype.prepareBuffer = function (length) {
        var spaceLeft = this._buffer.length - this.length;
        if (length > spaceLeft) {
            var spaceNeeded = length - spaceLeft + this.paddingSize;
            this._buffer = Buffer.concat([this._buffer, Buffer.alloc(spaceNeeded)], this._buffer.length + spaceNeeded);
        }
    };
    return ByteList;
}());
exports.ByteList = ByteList;
